@@ -42,8 +42,11 @@ DEFAULT_TEST_COMMAND_SETTLE_TIME = 1    #how long we wait after starting the tes
                             #this is the default value, some tests may override this below
 
 TEST_XPRA = True
-TEST_VNC = True
-USE_IPTABLES = True         #this requires iptables to be setup so we can use it for accounting
+TEST_VNC = False
+USE_IPTABLES = False        #this requires iptables to be setup so we can use it for accounting
+USE_VIRTUALGL = True        #allows us to run GL games and benchmarks using the GPU
+PREVENT_SLEEP = True
+PREVENT_SLEEP_COMMAND = ["xdotool", "keydown", "Shift_L", "keyup", "Shift_L"]
 
 LIMIT_TESTS = 2
 LIMIT_TESTS = 99999         #to limit the total number of tests being run
@@ -65,9 +68,10 @@ TRICKLE_BIN = "/usr/bin/trickle"
 TCBENCH = "/opt/VirtualGL/bin/tcbench"
 TCBENCH_LOG = "./tcbench.log"
 XORG_BIN = "/usr/local/bin/Xorg"
+VGLRUN_BIN = "/usr/bin/vglrun"
 
 #the glx tests:
-GLX_SPHERES = ["/opt/VirtualGL/bin/glxspheres64"]
+GLX_SPHERES = ["/usr/bin/glxspheres"]
 GLX_GEARS = ["/usr/bin/glxgears", "-geometry", "1240x900"]
 GLX_TESTS = [GLX_SPHERES, GLX_GEARS]
 
@@ -81,7 +85,11 @@ X11_TESTS = [X11_PERF, XTERM_TEST, FAKE_CONSOLE_USER_TEST, GTKPERF_TEST]
 #the screensaver tests:
 XSCREENSAVERS_PATH = "/usr/libexec/xscreensaver"
 def screensaver(x):
-    return  ["%s/%s" % (XSCREENSAVERS_PATH, x)]
+    for d in [os.path.join(sys.prefix, "bin"), XSCREENSAVERS_PATH, "/usr/bin", "/usr/local/bin"]:
+        f = os.path.join(d, x)
+        if os.path.exists(f) and os.path.isfile(f):
+            return f
+    return  None
 ALL_SCREENSAVER_TESTS = [screensaver(x) for x in
                             ["rss-glx-lattice", "rss-glx-plasma", "deluxe", "eruption", "memscroller", "moebiusgears", "polytopes"]
                          ]
@@ -100,23 +108,53 @@ TEST_COMMAND_SETTLE_TIME[NEXUIZ_TEST[0]] = 10
 TEST_COMMAND_SETTLE_TIME[XONOTIC_TEST[0]] = 20
 GAMES_TESTS = [NEXUIZ_TEST, XONOTIC_TEST]
 
+#sound and video tests:
+VIDEO_TESTS = []
+SOUND_TESTS = []
+VLC_BIN = "/usr/bin/vlc"
+MPLAYER_BIN = "/usr/bin/mplayer"
+
+MPLAYER_SOUND_LOOP_TEST = "while true; do %s ./test.mp3; done" % MPLAYER_BIN
+VLC_SOUND_TEST = (VLC_BIN, "-L", "--audio-visual=visual", "./test.mp3")
+if not os.path.exists("test.mp3"):
+    print("test.mp3 not found, the corresponding sound mplayer sound and vlc video tests are disabled")
+else:
+    SOUND_TESTS.append(MPLAYER_SOUND_LOOP_TEST)
+    VIDEO_TESTS.append(VLC_SOUND_TEST)
+
+#video tests
+VLC_VIDEO_TEST = (VLC_BIN, "-L", "./test.avi")
+MPLAYER_VIDEO_TEST = "while true; do %s test.avi; done" % MPLAYER_BIN
+if not os.path.exists("test.avi"):
+    print("test.avi not found, vlc and mplayer video tests are disabled")
+else:
+    VIDEO_TESTS.append(VLC_VIDEO_TEST)
+    VIDEO_TESTS.append(MPLAYER_VIDEO_TEST)
+
 #our selection:
 TEST_CANDIDATES = [screensaver("deluxe")]
 TEST_CANDIDATES = X11_TESTS + SOME_SCREENSAVER_TESTS + GAMES_TESTS
 TEST_CANDIDATES = GLX_TESTS + X11_TESTS + ALL_SCREENSAVER_TESTS + GAMES_TESTS
-TEST_CANDIDATES = GLX_TESTS + X11_TESTS + ALL_SCREENSAVER_TESTS
+TEST_CANDIDATES = GLX_TESTS + X11_TESTS + ALL_SCREENSAVER_TESTS + SOUND_TESTS + VIDEO_TESTS
 
 
 #now we filter all the test commands and only keep the valid ones:
 print("Checking for test commands:")
 X11_TEST_COMMANDS = []
 for x in TEST_CANDIDATES:
-    if x!=GTKPERF_TEST and not os.path.exists(x[0]):
+    if x is None:
+        continue
+    if type(x) in (list, tuple) and not os.path.exists(x[0]):
         print("* WARNING: cannot find %s - removed from tests" % str(x))
     else:
         print("* adding test: %s" % str(x))
         X11_TEST_COMMANDS.append(x)
-TEST_NAMES = {GTKPERF_TEST: "gtkperf"}
+TEST_NAMES = {GTKPERF_TEST: "gtkperf",
+              MPLAYER_SOUND_LOOP_TEST : "mplayer sound",
+              VLC_SOUND_TEST : "vlc sound visual",
+              MPLAYER_VIDEO_TEST : "mplayer video",
+              VLC_VIDEO_TEST : "vlc video",
+              }
 
 
 XVNC_BIN = "/usr/bin/Xvnc"
@@ -151,7 +189,7 @@ XPRA_SERVER_STOP_COMMANDS = [
                              "ps -ef | grep -i [X]org-for-Xpra-:%s | awk '{print $2}' | xargs kill" % DISPLAY_NO
                              ]
 XPRA_INFO_COMMAND = [XPRA_BIN, "info", "tcp:%s:%s" % (IP, PORT)]
-XPRA_USE_XDUMMY = True
+XPRA_FORCE_XDUMMY = False
 XPRA_QUALITY_OPTIONS = [40, 90]
 XPRA_QUALITY_OPTIONS = [80]
 XPRA_QUALITY_OPTIONS = [10, 40, 80, 90]
@@ -172,6 +210,25 @@ XPRA_ENCODING_QUALITY_OPTIONS = {"jpeg" : XPRA_QUALITY_OPTIONS,
                                  "webp" : XPRA_QUALITY_OPTIONS,
                                  "x264" : XPRA_QUALITY_OPTIONS+[-1],
                                  }
+XPRA_OPENGL_OPTIONS = {"x264" : [True, False],
+                       "vpx" : [True, False] }
+
+
+XPRA_SPEAKER_OPTIONS = [None]
+XPRA_MICROPHONE_OPTIONS = [None]
+TEST_SOUND = False
+if TEST_SOUND:
+    from xpra.sound.gstreamer_util import CODEC_ORDER, has_codec
+    if XPRA_VERSION_NO>=[0, 9]:
+        #0.9 onwards supports all codecs defined:
+        XPRA_SPEAKER_OPTIONS = [x for x in CODEC_ORDER if has_codec(x)]
+    elif XPRA_VERSION_NO==[0, 8]:
+        #only mp3 works in 0.8:
+        XPRA_SPEAKER_OPTIONS = ["mp3"]
+    else:
+        #none before that
+        XPRA_SPEAKER_OPTIONS = [None]
+
 
 password_filename = "./test-password.txt"
 try:
@@ -227,7 +284,9 @@ def getoutput_lines(cmd, pattern, setup_info):
 
 def getoutput_line(cmd, pattern, setup_info):
     lines = getoutput_lines(cmd, pattern, setup_info)
-    assert len(lines)==1, "expected 1 line matching '%s' from %s but found %s" % (pattern, cmd, len(lines))
+    if len(lines)!=1:
+        print("WARNING: expected 1 line matching '%s' from %s but found %s" % (pattern, cmd, len(lines)))
+        return "not found"
     return  lines[0]
 
 def get_cpu_info():
@@ -416,7 +475,9 @@ def measure_client(server_pid, name, cmd, get_stats_cb):
 def with_server(start_server_command, stop_server_commands, in_tests, get_stats_cb):
     tests = in_tests[:LIMIT_TESTS]
     print("going to run %s tests: %s" % (len(tests), [x[0] for x in tests]))
+    print("*******************************************")
     print("ETA: %s minutes" % int((SERVER_SETTLE_TIME+DEFAULT_TEST_COMMAND_SETTLE_TIME+SETTLE_TIME+MEASURE_TIME+1)*len(tests)/60))
+    print("*******************************************")
 
     server_process = None
     test_command_process = None
@@ -425,7 +486,7 @@ def with_server(start_server_command, stop_server_commands, in_tests, get_stats_
     errors = 0
     results = []
     count = 0
-    for name, tech_name, server_version, client_version, encoding, compression, encryption, ssh, (down,up,latency), test_command, client_cmd in tests:
+    for name, tech_name, server_version, client_version, encoding, opengl, compression, encryption, ssh, (down,up,latency), test_command, client_cmd in tests:
         try:
             print("**************************************************************")
             count += 1
@@ -449,15 +510,29 @@ def with_server(start_server_command, stop_server_commands, in_tests, get_stats_
 
                 try:
                     #start the test command:
-                    print("starting test command: %s, settle time=%s" % (str(test_command), test_command_settle_time))
-                    test_command_process = subprocess.Popen(test_command, stdin=None, stdout=subprocess.PIPE, stderr=subprocess.PIPE, env=env, shell=type(test_command)==str)
+                    if USE_VIRTUALGL:
+                        if type(test_command)==str:
+                            cmd = VGLRUN_BIN + test_command
+                        elif type(test_command)==list:
+                            cmd = [VGLRUN_BIN] + test_command
+                        else:
+                            raise Exception("invalid test command type: %s for %s" % (type(test_command), test_command))
+                    else:
+                        cmd = test_command
+                        
+                    print("starting test command: %s, settle time=%s" % (str(cmd), test_command_settle_time))
+                    test_command_process = subprocess.Popen(test_command, stdin=None, stdout=subprocess.PIPE, stderr=subprocess.PIPE, env=env, shell=type(cmd)==str)
+
+                    if PREVENT_SLEEP:
+                        subprocess.Popen(PREVENT_SLEEP_COMMAND)
+                    
                     time.sleep(test_command_settle_time)
                     code = test_command_process.poll()
                     assert code is None, "test command %s failed to start: exit code is %s" % (test_command, code)
 
                     #run the client test
                     result = [name, tech_name, server_version, client_version, " ".join(sys.argv[1:]), SVN_VERSION]
-                    result += [encoding, get_command_name(test_command)]
+                    result += [encoding, opengl, get_command_name(test_command)]
                     result += [MEASURE_TIME, time.time(), CPU_INFO, PLATFORM, KERNEL_VERSION, XORG_VERSION, OPENGL_INFO, WINDOW_MANAGER]
                     result += ["%sx%s" % gdk.get_default_root_window().get_size()]
                     result += [compression, encryption, ssh, down, up, latency]
@@ -602,7 +677,7 @@ def xpra_get_stats(last_record=None):
 
 def get_xpra_start_server_command():
     cmd = [XPRA_BIN, "--no-daemon", "--bind-tcp=0.0.0.0:%s" % PORT]
-    if XPRA_USE_XDUMMY:
+    if XPRA_FORCE_XDUMMY:
         cmd.append("--xvfb=%s -nolisten tcp +extension GLX +extension RANDR +extension RENDER -logfile %s -config %s" % (XORG_BIN, XORG_LOG, XORG_CONFIG))
     if XPRA_VERSION_NO>=[0, 5]:
         cmd.append("--no-notifications")
@@ -623,45 +698,67 @@ def test_xpra():
         for down,up,latency in shaping_options:
             for x11_test_command in X11_TEST_COMMANDS:
                 for encoding in XPRA_TEST_ENCODINGS:
-                    quality_options = XPRA_ENCODING_QUALITY_OPTIONS.get(encoding, [-1])
-                    for quality in quality_options:
-                        comp_options = XPRA_COMPRESSION_OPTIONS
-                        for compression in comp_options:
-                            cmd = trickle_command(down, up, latency)
-                            cmd += [XPRA_BIN, "attach"]
-                            if connect_option=="ssh":
-                                cmd.append("ssh:%s:%s" % (IP, DISPLAY_NO))
-                            elif connect_option=="tcp":
-                                cmd.append("tcp:%s:%s" % (IP, PORT))
-                            else:
-                                cmd.append(":%s" % (DISPLAY_NO))
-                            cmd.append("--readonly")
-                            cmd.append("--password-file=%s" % password_filename)
-                            if compression is not None:
-                                cmd += ["-z", str(compression)]
-                            if XPRA_VERSION_NO>=[0, 3]:
-                                cmd.append("--enable-pings")
-                                cmd.append("--no-clipboard")
-                            if XPRA_VERSION_NO>=[0, 5]:
-                                cmd.append("--no-bell")
-                                cmd.append("--no-cursors")
-                                cmd.append("--no-notifications")
-                            if XPRA_VERSION_NO>=[0, 8] and encryption:
-                                cmd.append("--encryption=%s" % encryption)
-                            if encoding in ("jpeg", "webp"):
-                                if XPRA_VERSION_NO>=[0, 7]:
-                                    cmd.append("--quality=%s" % quality)
-                                else:
-                                    cmd.append("--jpeg-quality=%s" % quality)
-                                name = "%s-%s" % (encoding, quality)
-                            else:
-                                name = encoding
-                            if encoding!="mmap":
-                                cmd.append("--no-mmap")
-                                cmd.append("--encoding=%s" % encoding)
-                            command_name = get_command_name(x11_test_command)
-                            test_name = "%s (%s - %s - %s - %s - via %s)" % (name, command_name, compression, encryption, trickle_str(down, up, latency), connect_option)
-                            tests.append((test_name, "xpra", XPRA_VERSION, XPRA_VERSION, encoding, compression, encryption, connect_option, (down,up,latency), x11_test_command, cmd))
+                    if XPRA_VERSION_NO>=[0, 9]:
+                        opengl_options = XPRA_OPENGL_OPTIONS.get(encoding, [False])
+                    else:
+                        opengl_options = [False]
+                    for opengl in opengl_options:
+                        quality_options = XPRA_ENCODING_QUALITY_OPTIONS.get(encoding, [-1])
+                        for quality in quality_options:
+                            for speaker in XPRA_SPEAKER_OPTIONS:
+                                for mic in XPRA_MICROPHONE_OPTIONS:
+                                    comp_options = XPRA_COMPRESSION_OPTIONS
+                                    for compression in comp_options:
+                                        cmd = trickle_command(down, up, latency)
+                                        cmd += [XPRA_BIN, "attach"]
+                                        if connect_option=="ssh":
+                                            cmd.append("ssh:%s:%s" % (IP, DISPLAY_NO))
+                                        elif connect_option=="tcp":
+                                            cmd.append("tcp:%s:%s" % (IP, PORT))
+                                        else:
+                                            cmd.append(":%s" % (DISPLAY_NO))
+                                        cmd.append("--readonly")
+                                        cmd.append("--password-file=%s" % password_filename)
+                                        if compression is not None:
+                                            cmd += ["-z", str(compression)]
+                                        if XPRA_VERSION_NO>=[0, 3]:
+                                            cmd.append("--enable-pings")
+                                            cmd.append("--no-clipboard")
+                                        if XPRA_VERSION_NO>=[0, 5]:
+                                            cmd.append("--no-bell")
+                                            cmd.append("--no-cursors")
+                                            cmd.append("--no-notifications")
+                                        if XPRA_VERSION_NO>=[0, 8] and encryption:
+                                            cmd.append("--encryption=%s" % encryption)
+                                        if encoding in ("jpeg", "webp"):
+                                            if XPRA_VERSION_NO>=[0, 7]:
+                                                cmd.append("--quality=%s" % quality)
+                                            else:
+                                                cmd.append("--jpeg-quality=%s" % quality)
+                                            name = "%s-%s" % (encoding, quality)
+                                        else:
+                                            name = encoding
+                                        if speaker is None:
+                                            if XPRA_VERSION_NO>=[0, 8]:
+                                                cmd.append("--no-speaker")
+                                        else:
+                                            cmd.append("--speaker-codec=%s" % speaker)
+                                        if mic is None:
+                                            if XPRA_VERSION_NO>=[0, 8]:
+                                                cmd.append("--no-microphone")
+                                        else:
+                                            cmd.append("--microphone-codec=%s" % mic)
+                                        if encoding!="mmap":
+                                            cmd.append("--no-mmap")
+                                            cmd.append("--encoding=%s" % encoding)
+                                        if XPRA_VERSION_NO>=[0, 9]:
+                                            cmd.append("--opengl=%s" % opengl)
+                                        command_name = get_command_name(x11_test_command)
+                                        test_name = "%s (%s - %s - %s - %s - via %s)" % \
+                                            (name, command_name, compression, encryption, trickle_str(down, up, latency), connect_option)
+                                        tests.append((test_name, "xpra", XPRA_VERSION, XPRA_VERSION, \
+                                                      encoding, opengl, compression, encryption, connect_option, \
+                                                      (down,up,latency), x11_test_command, cmd))
     return with_server(get_xpra_start_server_command(), XPRA_SERVER_STOP_COMMANDS, tests, xpra_get_stats)
 
 
@@ -796,8 +893,11 @@ def test_vnc():
                             else:
                                 zlibtxt = "zlib=%s" % zlib
                             command_name = get_command_name(x11_test_command)
-                            test_name = "vnc (%s - %s - %s - compression=%s - %s - %s)" % (command_name, encoding, zlibtxt, compression, jpegtxt, trickle_str(down, up, latency))
-                            tests.append((test_name, "vnc", XVNC_VERSION, VNCVIEWER_VERSION, encoding, compression, None, False, (down,up,latency), x11_test_command, cmd))
+                            test_name = "vnc (%s - %s - %s - compression=%s - %s - %s)" % \
+                                        (command_name, encoding, zlibtxt, compression, jpegtxt, trickle_str(down, up, latency))
+                            tests.append((test_name, "vnc", XVNC_VERSION, VNCVIEWER_VERSION, \
+                                          encoding, False, compression, None, False, \
+                                          (down,up,latency), x11_test_command, cmd))
     return with_server(XVNC_SERVER_START_COMMAND, XVNC_SERVER_STOP_COMMANDS, tests, get_vnc_stats)
 
 
@@ -816,7 +916,7 @@ def main():
     print("RESULTS:")
     print("")
     headers = ["Test Name", "Remoting Tech", "Server Version", "Client Version", "Custom Params", "SVN Version",
-               "Encoding", "Test Command", "Sample Duration (s)", "Sample Time (epoch)",
+               "Encoding", "OpenGL", "Test Command", "Sample Duration (s)", "Sample Time (epoch)",
                "CPU info", "Platform", "Kernel Version", "Xorg version", "OpenGL", "Client Window Manager", "Screen Size",
                "Compression", "Encryption", "Connect via", "download limit (KB)", "upload limit (KB)", "latency (ms)",
                "packets in/s", "packets in: bytes/s", "packets out/s", "packets out: bytes/s"]
