@@ -58,6 +58,13 @@ if gtk.pygtk_version<(2,17):
     log.error("your version of PyGTK is too old - expect some bugs")
 
 
+#if you want to use a virtual screen bigger than 32767x32767
+#you will need to change those values, but some broken toolkits
+#will then misbehave (they use signed shorts instead of signed ints..)
+MAX_WINDOW_SIZE = 2**15-1
+MAX_ASPECT = 2*15-1
+
+
 # Todo:
 #   client focus hints
 #   _NET_WM_SYNC_REQUEST
@@ -345,9 +352,24 @@ class BaseWindowModel(AutoPropGObjectMixin, gobject.GObject):
             self._composite.do_wimpiggy_configure_event(event)
 
     def do_get_property_geometry(self, pspec):
-        (x, y, w, h, b) = self._geometry
+        if self._geometry is None:
+            self.read_geometry(False)
+        x, y, w, h, b = self._geometry
         return (x, y, w + 2*b, h + 2*b)
 
+    def read_geometry(self, emit):
+        def synced_update():
+            xwin = get_xwindow(self.client_window)
+            self._geometry = geometry_with_border(xwin)
+            log("BaseWindowModel.read_geometry(%s) geometry(%s)=%s", emit, hex(xwin), self._geometry)
+            if emit:
+                self.emit("geometry")
+        try:
+            trap.call_unsynced(synced_update)
+        except XError:
+            log.error("failed to retrieve updated window geometry - maybe it's gone?", exc_info=True)
+
+
     def unmanage(self, exiting=False):
         if self._managed:
             self.emit("unmanaged", exiting)
@@ -425,12 +447,9 @@ class OverrideRedirectWindowModel(BaseWindowModel):
 
     def composite_configure_event(self, composite_window, event):
         BaseWindowModel.composite_configure_event(self, composite_window, event)
-        log("OverrideRedirectWindowModel.composite_configure_event(%s, %s) client window geometry=%s", composite_window, event, self.client_window.get_geometry())
-        try:
-            self._geometry = trap.call_unsynced(geometry_with_border, self.client_window)
-            self.emit("geometry")
-        except XError:
-            log.error("failed to update geometry!", exc_info=True)
+        if self.client_window is None:
+            return
+        self.read_geometry(True)
 
     def _guess_window_type(self, transient_for):
         return "_NET_WM_WINDOW_TYPE_NORMAL"
@@ -644,7 +663,7 @@ class WindowModel(BaseWindowModel):
         hints = self.get_property("size-hints")
         self._sanitize_size_hints(hints)
         nw, nh = calc_constrained_size(w, h, hints)[:2]
-        if nw>=32768 or nh>=32768:
+        if nw>=MAX_WINDOW_SIZE or nh>=MAX_WINDOW_SIZE:
             #we can't handle windows that big!
             raise Unmanageable("window constrained size is too large: %sx%s (from client geometry: %s,%s with size hints=%s)" % (nw, nh, w, h, hints))
         log("setup() resizing windows to %sx%s", nw, nh)
@@ -783,7 +802,7 @@ class WindowModel(BaseWindowModel):
                     f = float(v)
                 except:
                     f = None
-                if f is None or f>=(2**32-1):
+                if f is None or f>=MAX_ASPECT:
                     log.warn("clearing invalid aspect hint value for %s: %s", attr, v)
                     setattr(size_hints, attr, -1.0)
         for attr in ["max_size", "min_size", "base_size", "resize_inc",
@@ -794,7 +813,7 @@ class WindowModel(BaseWindowModel):
                     w,h = v
                 except:
                     w,h = None,None
-                if (w is None or h is None) or w>=(2**32-1) or h>(2**32-1):
+                if (w is None or h is None) or w>=MAX_WINDOW_SIZE or h>=MAX_WINDOW_SIZE:
                     log.warn("clearing invalid size hint value for %s: %s", attr, v)
                     setattr(size_hints, attr, (-1,-1))
         #if max-size is smaller than min-size (bogus), clamp it..
@@ -804,9 +823,9 @@ class WindowModel(BaseWindowModel):
             minw,minh = mins
             maxw,maxh = maxs
             if maxw<minw or maxh<minh:
-                size_hints.max_size = max(minw, maxw), max(minh, maxh)
-                log.warn("invalid max_size=%s for min_size=%s has now been clamped to: %s",
-                         maxs, mins, size_hints.max_size)
+                size_hints.max_size = None
+                log.warn("invalid max_size=%s for min_size=%s has now been cleared",
+                         maxs, mins)
 
     def _update_client_geometry(self):
         owner = self.get_property("owner")
