@@ -16,7 +16,6 @@ import atexit
 import signal
 import socket
 
-from xpra.wait_for_x_server import wait_for_x_server        #@UnresolvedImport
 from xpra.dotxpra import DotXpra, ServerSockInUse
 
 o0117 = 79
@@ -52,24 +51,27 @@ def deadly_signal(signum, frame):
 # child to exit and us to receive the SIGCHLD before our fork() returns (and
 # thus before we even know the pid of the child).  So be careful:
 class ChildReaper(object):
-    def __init__(self, quit_cb, children_pids):
+    def __init__(self, quit_cb, children_pids={}):
         self._quit = quit_cb
         self._children_pids = children_pids
         self._dead_pids = set()
+        from wimpiggy.log import Logger
+        self._logger = Logger()
 
     def check(self):
-        if (self._children_pids
-            and self._children_pids.issubset(self._dead_pids)):
-            from wimpiggy.log import Logger
-            log = Logger()
-            log.info("all children have exited and --exit-with-children was specified, exiting")
-            self._quit()
+        if self._children_pids:
+            pids = set(self._children_pids.keys())
+            if pids.issubset(self._dead_pids):
+                self._quit()
 
     def sigchld(self, signum, frame):
         self.reap()
 
     def add_dead_pid(self, pid):
         if pid not in self._dead_pids:
+            cmd = self._children_pids.get(pid)
+            if cmd:
+                self._logger.info("child '%s' with pid %s has terminated", cmd, pid)
             self._dead_pids.add(pid)
             self.check()
 
@@ -115,7 +117,7 @@ def xpra_runner_shell_script(xpra_file, starting_dir, socket_dir):
         # as they are either irrelevant or simply do not match
         # the new environment used by xpra
         # TODO: use a whitelist
-        if var in ["XDG_SESSION_COOKIE", "LS_COLORS"]:
+        if var in ["XDG_SESSION_COOKIE", "LS_COLORS", "DISPLAY"]:
             continue
         #XPRA_SOCKET_DIR is a special case, it is handled below
         if var=="XPRA_SOCKET_DIR":
@@ -169,12 +171,23 @@ def create_unix_domain_socket(sockpath, mmap_group):
 def create_tcp_socket(parser, spec):
     if ":" not in spec:
         parser.error("TCP port must be specified as [HOST]:PORT")
-    (host, port) = spec.split(":", 1)
+    (host, port) = spec.rsplit(":", 1)
     if host == "":
         host = "127.0.0.1"
-    listener = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+    try:
+        iport = int(port)
+    except:
+        raise Exception("invalid port number: %s" % port)
+    if host.find(":")<0:
+        listener = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        sockaddr = (host, iport)
+    else:
+        assert socket.has_ipv6, "specified an IPv6 address but this is not supported"
+        res = socket.getaddrinfo(host, iport, socket.AF_INET6, socket.SOCK_STREAM, 0, socket.SOL_TCP)
+        listener = socket.socket(socket.AF_INET6, socket.SOCK_STREAM)
+        sockaddr = res[0][-1]
     listener.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
-    listener.bind((host, int(port)))
+    listener.bind(sockaddr)
     return listener
 
 def close_all_fds(exceptions=[]):
@@ -215,7 +228,7 @@ def run_server(parser, opts, mode, xpra_file, extra_args):
         except:
             pass
 
-    if opts.exit_with_children and not opts.children:
+    if not shadowing and opts.exit_with_children and not opts.start_child:
         sys.stderr.write("--exit-with-children specified without any children to spawn; exiting immediately")
         return  1
 
@@ -223,8 +236,7 @@ def run_server(parser, opts, mode, xpra_file, extra_args):
     signal.signal(signal.SIGINT, deadly_signal)
     signal.signal(signal.SIGTERM, deadly_signal)
 
-    from xpra.scripts.main import get_default_socket_dir
-    dotxpra = DotXpra(opts.sockdir or get_default_socket_dir())
+    dotxpra = DotXpra(opts.socket_dir)
 
     # This used to be given a display-specific name, but now we give it a
     # single fixed name and if multiple servers are started then the last one
@@ -242,7 +254,14 @@ def run_server(parser, opts, mode, xpra_file, extra_args):
 
     # Daemonize:
     if opts.daemon:
-        logpath = dotxpra.log_path(display_name) + ".log"
+        if opts.log_file:
+            if os.path.isabs(opts.log_file):
+                logpath = opts.log_file
+            else:
+                logpath = os.path.join(dotxpra.sockdir(), opts.log_file)
+            logpath = logpath.replace("$DISPLAY", display_name)
+        else:
+            logpath = dotxpra.log_path(display_name) + ".log"
         sys.stderr.write("Entering daemon mode; "
                          + "any further errors will be reported to:\n"
                          + ("  %s\n" % logpath))
@@ -280,7 +299,7 @@ def run_server(parser, opts, mode, xpra_file, extra_args):
         os.fchmod(scriptfile.fileno(), o0700 & ~umask)
     else:
         os.chmod(scriptpath, o0700 & ~umask)
-    scriptfile.write(xpra_runner_shell_script(xpra_file, starting_dir, opts.sockdir))
+    scriptfile.write(xpra_runner_shell_script(xpra_file, starting_dir, opts.socket_dir))
     scriptfile.close()
 
     from wimpiggy.log import Logger
@@ -290,39 +309,68 @@ def run_server(parser, opts, mode, xpra_file, extra_args):
     # That way, errors won't make us kill the Xvfb
     # (which may not be ours to kill at that point)
     sockets = []
-    if opts.bind_tcp:
-        try:
-            tcp_socket = create_tcp_socket(parser, opts.bind_tcp)
-            sockets.append(tcp_socket)
-            def cleanup_tcp_socket():
-                log.info("closing tcp socket %s", opts.bind_tcp)
-                try:
-                    tcp_socket.close()
-                except:
-                    pass
-            _cleanups.append(cleanup_tcp_socket)
-        except Exception, e:
-            log.error("cannot start - failed to create tcp socket at %s: %s" % (opts.bind_tcp, e))
-            return  1
-    #print("creating server socket %s" % sockpath)
+    if opts.bind_tcp and len(opts.bind_tcp):
+        def setup_tcp_socket(bind_to):
+            try:
+                tcp_socket = create_tcp_socket(parser, bind_to)
+                sockets.append(tcp_socket)
+                def cleanup_tcp_socket():
+                    log.info("closing tcp socket %s", bind_to)
+                    try:
+                        tcp_socket.close()
+                    except:
+                        pass
+                _cleanups.append(cleanup_tcp_socket)
+            except Exception, e:
+                log.error("cannot start - failed to create tcp socket at %s : %s" % (bind_to, e))
+                return  1
+        for tcp_s in set(opts.bind_tcp):
+            setup_tcp_socket(tcp_s)
+
     clobber = upgrading or opts.use_display
-    try:
-        sockpath = dotxpra.server_socket_path(display_name, clobber, wait_for_unknown=5)
-    except ServerSockInUse:
-        parser.error("You already have an xpra server running at %s\n"
-                     "  (did you want 'xpra upgrade'?)"
-                     % (display_name,))
-    sockets.append(create_unix_domain_socket(sockpath, opts.mmap_group))
-    def cleanup_socket():
-        log.info("removing socket %s", sockpath)
+    if not sys.platform.startswith("win"):
+        #print("creating server socket %s" % sockpath)
         try:
-            os.unlink(sockpath)
-        except:
-            pass
-    _cleanups.append(cleanup_socket)
+            sockpath = dotxpra.server_socket_path(display_name, clobber, wait_for_unknown=5)
+        except ServerSockInUse:
+            parser.error("You already have an xpra server running at %s\n"
+                         "  (did you want 'xpra upgrade'?)"
+                         % (display_name,))
+        sockets.append(create_unix_domain_socket(sockpath, opts.mmap_group))
+        def cleanup_socket():
+            log.info("removing socket %s", sockpath)
+            try:
+                os.unlink(sockpath)
+            except:
+                pass
+        _cleanups.append(cleanup_socket)
 
     # Do this after writing out the shell script:
     os.environ["DISPLAY"] = display_name
+    def unsetenv(*varnames):
+        for x in varnames:
+            if os.environ.get(x):
+                del os.environ[x]
+    #we don't want client apps to think these mean anything:
+    #(if set, they belong to the desktop the server was started from)
+    #TODO: simply whitelisting the env would be safer/better
+    unsetenv("DESKTOP_SESSION",
+             "GDMSESSION",
+             "SESSION_MANAGER",
+             "XDG_VTNR",
+             "XDG_MENU_PREFIX",
+             "XDG_SEAT",
+             #"XDG_RUNTIME_DIR",
+             "QT_GRAPHICSSYSTEM_CHECKED",
+             )
+    #force 'simple' / 'xim', as 'ibus' 'immodule' breaks keyboard handling
+    #unless its daemon is also running - and we don't know if it is..
+    #this should override any XSETTINGS too.
+    os.environ["DISABLE_IMSETTINGS"] = "true"
+    os.environ["GTK_IM_MODULE"] = "xim"                     #or "gtk-im-context-simple"?
+    os.environ["QT_IM_MODULE"] = "xim"                      #or "simple"?
+    os.environ["IMSETTINGS_MODULE"] = "none"                #or "xim"?
+    os.environ["XMODIFIERS"] = ""
 
     if not clobber and not shadowing:
         # We need to set up a new server environment
@@ -362,36 +410,44 @@ def run_server(parser, opts, mode, xpra_file, extra_args):
             return False
         if xvfb.poll() is None:
             return False
-        log.error("\n")
-        log.error("Xvfb command has terminated! xpra cannot continue\n")
-        log.error("\n")
+        log.error("")
+        log.error("Xvfb command has terminated! xpra cannot continue")
+        log.error("")
         if instance_exists:
-            log.error("There is an X11 server already running on display %s:\n" % display_name)
-            log.error("You may want to use:\n")
-            log.error("  'xpra upgrade' if an instance of xpra is still connected to it\n")
-            log.error("  'xpra --use-display start' to connect xpra to an existing X11 server only\n")
+            log.error("There is an X11 server already running on display %s:" % display_name)
+            log.error("You may want to use:")
+            log.error("  'xpra upgrade %s' if an instance of xpra is still connected to it" % display_name)
+            log.error("  'xpra --use-display start %s' to connect xpra to an existing X11 server only" % display_name)
+            log.error("")
         return True
 
     if xvfb_error():
         return  1
-    # Whether we spawned our server or not, it is now running -- or at least
-    # starting.  First wait for it to start up:
-    try:
-        wait_for_x_server(display_name, 3) # 3s timeout
-    except Exception, e:
-        sys.stderr.write("%s\n" % e)
-        return  1
-    if xvfb_error(True):
-        return  1
-    # Now we can safely load gtk and connect:
-    assert "gtk" not in sys.modules
-    import gtk
-    display = gtk.gdk.Display(display_name)
-    manager = gtk.gdk.display_manager_get()
-    default_display = manager.get_default_display()
-    if default_display is not None:
-        default_display.close()
-    manager.set_default_display(display)
+
+    if not sys.platform.startswith("win"):
+        from xpra.wait_for_x_server import wait_for_x_server        #@UnresolvedImport
+        # Whether we spawned our server or not, it is now running -- or at least
+        # starting.  First wait for it to start up:
+        try:
+            wait_for_x_server(display_name, 3) # 3s timeout
+        except Exception, e:
+            sys.stderr.write("%s\n" % e)
+            return  1
+        if xvfb_error(True):
+            return  1
+        # Now we can safely load gtk and connect:
+        assert "gtk" not in sys.modules
+        import gtk.gdk          #@Reimport
+        gtk.gdk.threads_init()
+        display = gtk.gdk.Display(display_name)
+        manager = gtk.gdk.display_manager_get()
+        default_display = manager.get_default_display()
+        if default_display is not None:
+            default_display.close()
+        manager.set_default_display(display)
+    else:
+        assert "gtk" not in sys.modules
+        import gtk          #@Reimport
 
     if shadowing:
         xvfb_pid = None
@@ -404,19 +460,18 @@ def run_server(parser, opts, mode, xpra_file, extra_args):
         if xvfb_pid is not None:
             save_pid(xvfb_pid)
 
-    def kill_xvfb():
-        # Close our display(s) first, so the server dying won't kill us.
-        log.info("killing xvfb with pid %s" % xvfb_pid)
-        for display in gtk.gdk.display_manager_get().list_displays():
-            display.close()
-        os.kill(xvfb_pid, signal.SIGTERM)
-    if xvfb_pid is not None and not opts.use_display and not shadowing:
-        _cleanups.append(kill_xvfb)
-
     if shadowing:
-        from xpra.shadow_server import XpraShadowServer
-        app = XpraShadowServer(sockets, opts)
+        if sys.platform.startswith("win"):
+            from xpra.win32.shadow_server import XpraWin32ShadowServer
+            app = XpraWin32ShadowServer(sockets, opts)
+        else:
+            from xpra.shadow_server import XpraX11ShadowServer
+            app = XpraX11ShadowServer(sockets, opts)
     else:
+        #check for an existing window manager:
+        from wimpiggy.wm import wm_check
+        if not wm_check(display):
+            return 1
         try:
             from wimpiggy.lowlevel import displayHasXComposite     #@UnresolvedImport
             # This import is delayed because the module depends on gtk:
@@ -428,68 +483,89 @@ def run_server(parser, opts, mode, xpra_file, extra_args):
         if not displayHasXComposite(root):
             log.error("Xpra is a compositing manager, it cannot use a display which lacks the XComposite extension!")
             return 1
-
         app = XpraServer(clobber, sockets, opts)
 
-    children_pids = set()
+    #we got this far so the sockets have initialized and
+    #the server should be able to manage the display
+    #from now on, if we exit without upgrading we will also kill the Xvfb
+    def kill_xvfb():
+        # Close our display(s) first, so the server dying won't kill us.
+        log.info("killing xvfb with pid %s" % xvfb_pid)
+        for display in gtk.gdk.display_manager_get().list_displays():
+            display.close()
+        os.kill(xvfb_pid, signal.SIGTERM)
+    if xvfb_pid is not None and not opts.use_display and not shadowing:
+        _cleanups.append(kill_xvfb)
+
+    children_pids = {}
     def reaper_quit():
         if opts.exit_with_children:
-            app.quit(False)
-    child_reaper = ChildReaper(reaper_quit, children_pids)
-    procs = []
-    if sys.version_info < (2, 7) or sys.version_info[:2] == (3, 0):
-        POLL_DELAY = int(os.environ.get("XPRA_POLL_DELAY", 2))
-        log.warn("Warning: outdated/buggy version of Python (%s), switching to process polling every %s seconds to support 'exit-with-children'", ".".join(str(x) for x in sys.version_info), POLL_DELAY)
-        ChildReaper.processes = procs
-        def check_procs():
-            for proc in ChildReaper.processes:
-                if proc.poll() is not None:
-                    child_reaper.add_dead_pid(proc.pid)
-                    child_reaper.check()
-            ChildReaper.processes = [proc for proc in ChildReaper.processes if proc.poll() is None]
-            return True
-        gobject.timeout_add(POLL_DELAY*1000, check_procs)
-    else:
-        #with non-buggy python, we can just check the list of pids
-        #whenever we get a SIGCHLD
-        signal.signal(signal.SIGCHLD, child_reaper.sigchld)
-        # Check once after the mainloop is running, just in case the exit
-        # conditions are satisfied before we even enter the main loop.
-        # (Programming with unix the signal API sure is annoying.)
-        def check_once():
-            child_reaper.check()
-            return False # Only call once
-        gobject.timeout_add(0, check_once)
+            log.info("all children have exited and --exit-with-children was specified, exiting")
+            gobject.idle_add(app.clean_quit)
 
-    if not upgrading and not shadowing and opts.pulseaudio and len(opts.pulseaudio_command)>0:
-        pa_proc = subprocess.Popen(opts.pulseaudio_command, shell=True, close_fds=True)
-        procs.append(pa_proc)
-        log.info("pulseaudio server started with pid %s", pa_proc.pid)
-        def check_pa_start():
-            if pa_proc.poll() is not None or pa_proc.pid in child_reaper._dead_pids:
-                log.warn("Warning: pulseaudio has terminated. Either fix the pulseaudio command line or use --no-pulseaudio to avoid this warning.")
-            return False
-        gobject.timeout_add(1000*2, check_pa_start)
-        def cleanup_pa():
-            if pa_proc.poll() is None and pa_proc.pid not in child_reaper._dead_pids:
-                log.info("stopping pulseaudio with pid %s", pa_proc.pid)
-                try:
-                    pa_proc.terminate()
-                except:
-                    pass
-        _cleanups.append(cleanup_pa)
+    procs = []
+    if os.name=="posix":
+        child_reaper = ChildReaper(reaper_quit, children_pids)
+        old_python = sys.version_info < (2, 7) or sys.version_info[:2] == (3, 0)
+        if old_python:
+            POLL_DELAY = int(os.environ.get("XPRA_POLL_DELAY", 2))
+            log.warn("Warning: outdated/buggy version of Python: %s", ".".join(str(x) for x in sys.version_info))
+            log.warn("switching to process polling every %s seconds to support 'exit-with-children'", POLL_DELAY)
+            ChildReaper.processes = procs
+            def check_procs():
+                for proc in ChildReaper.processes:
+                    if proc.poll() is not None:
+                        child_reaper.add_dead_pid(proc.pid)
+                        child_reaper.check()
+                ChildReaper.processes = [proc for proc in ChildReaper.processes if proc.poll() is None]
+                return True
+            gobject.timeout_add(POLL_DELAY*1000, check_procs)
+        else:
+            #with non-buggy python, we can just check the list of pids
+            #whenever we get a SIGCHLD
+            signal.signal(signal.SIGCHLD, child_reaper.sigchld)
+            # Check once after the mainloop is running, just in case the exit
+            # conditions are satisfied before we even enter the main loop.
+            # (Programming with unix the signal API sure is annoying.)
+            def check_once():
+                child_reaper.check()
+                return False # Only call once
+            gobject.timeout_add(0, check_once)
+
+        log("upgrading=%s, shadowing=%s, pulseaudio=%s, pulseaudio_command=%s",
+                 upgrading, shadowing, opts.pulseaudio, opts.pulseaudio_command)
+        if not upgrading and not shadowing and opts.pulseaudio and len(opts.pulseaudio_command)>0:
+            pa_proc = subprocess.Popen(opts.pulseaudio_command, shell=True, close_fds=True)
+            procs.append(pa_proc)
+            log.info("pulseaudio server started with pid %s", pa_proc.pid)
+            def check_pa_start():
+                if pa_proc.poll() is not None or pa_proc.pid in child_reaper._dead_pids:
+                    log.warn("Warning: pulseaudio has terminated. Either fix the pulseaudio command line or use --no-pulseaudio to avoid this warning.")
+                return False
+            gobject.timeout_add(1000*2, check_pa_start)
+            def cleanup_pa():
+                log("cleanup_pa() process.poll()=%s, pid=%s, dead_pids=%s", pa_proc.poll(), pa_proc.pid, child_reaper._dead_pids)
+                if pa_proc.poll() is None and pa_proc.pid not in child_reaper._dead_pids:
+                    log.info("stopping pulseaudio with pid %s", pa_proc.pid)
+                    try:
+                        pa_proc.terminate()
+                    except:
+                        log.warn("error trying to stop pulseaudio", exc_info=True)
+            _cleanups.append(cleanup_pa)
     if opts.exit_with_children:
-        assert opts.children
-    if opts.children:
+        assert opts.start_child
+    if opts.start_child:
+        assert os.name=="posix"
         #disable ubuntu's global menu using env vars:
         env = os.environ.copy()
-        env["UBUNTU_MENUPROXY"] = ""
-        env["QT_X11_NO_NATIVE_MENUBAR"] = "1"
-        for child_cmd in opts.children:
+        if os.name=="posix":
+            env["UBUNTU_MENUPROXY"] = ""
+            env["QT_X11_NO_NATIVE_MENUBAR"] = "1"
+        for child_cmd in opts.start_child:
             if child_cmd:
                 try:
                     proc = subprocess.Popen(child_cmd, env=env, shell=True, close_fds=True)
-                    children_pids.add(proc.pid)
+                    children_pids[proc.pid] = child_cmd
                     procs.append(proc)
                 except OSError, e:
                     sys.stderr.write("Error spawning child '%s': %s\n"
