@@ -839,8 +839,12 @@ class ServerBase(object):
         return max_w, max_h
 
     def _get_desktop_size_capability(self, server_source, root_w, root_h):
+        from wimpiggy.window import MAX_WINDOW_SIZE
+        max_w, max_h = self.get_max_screen_size()
         client_size = server_source.desktop_size
         log("client resolution is %s, current server resolution is %sx%s", client_size, root_w, root_h)
+        if max_w>MAX_WINDOW_SIZE or max_h>MAX_WINDOW_SIZE:
+            log.warn("maximum size is very large: %sx%s, you may encounter window sizing problems", max_w, max_h)
         if not client_size:
             """ client did not specify size, just return what we have """
             return    root_w, root_h
